/**
 * vim: set ts=4 sw=4 et tw=99:
 * (C)2009 David Anderson, AlliedModders LLC
 */
#include "platform.h"

void
Profiler::Start()
{
#if defined WIN32
    QueryPerformanceCounter(&start);
#else
    gettimeofday(&start, NULL);
#endif
}

void
Profiler::Stop()
{
#if defined WIN32
    QueryPerformanceCounter(&stop);
#else
    gettimeofday(&stop, NULL);
#endif
}

double
Profiler::Time()
{
#if defined WIN32
    LARGE_INTEGER freq;
	
    QueryPerformanceFrequency(&freq);
    LONGLONG diff = stop.QuadPart - start.QuadPart;
    return double(diff) / double(freq.QuadPart);
#else
    double diff = double(stop.tv_sec - start.tv_sec);
    if (stop.tv_usec >= start.tv_usec)
    {
        diff += double(stop.tv_usec - start.tv_usec) / 1000000.0;
    }
    else
    {
        diff += 1.0;
        diff += double(stop.tv_usec) / 1000000.0;
        diff += double(1000000 - start.tv_usec) / 1000000.0;
    }
    return diff;
#endif
}

