#include <ApplicationServices/ApplicationServices.h>
#include "platform.h"

//#define BITMAP

CGContextRef MyCreatePDFContext (const CGRect *inMediaBox, CFStringRef path)
{
    CGContextRef myOutContext = NULL;
    CFURLRef url;

    url = CFURLCreateWithFileSystemPath (NULL,
                                path,
                                kCFURLPOSIXPathStyle,
                                false);

    if (url != NULL) {
        myOutContext = CGPDFContextCreateWithURL (url, inMediaBox, NULL);
        CFRelease(url);
    }

    return myOutContext;
}

CGContextRef MyCreateBitmapContext (int pixelsWide,
                            int pixelsHigh)
{
    CGContextRef    context = NULL;
    CGColorSpaceRef colorSpace;
    void *          bitmapData;
    int             bitmapByteCount;
    int             bitmapBytesPerRow;
    bitmapBytesPerRow   = (pixelsWide * 4);// 1
    bitmapByteCount     = (bitmapBytesPerRow * pixelsHigh);

    colorSpace = CGColorSpaceCreateWithName(kCGColorSpaceGenericRGB);// 2
    bitmapData = malloc( bitmapByteCount );// 3

    if (bitmapData == NULL)
    {
        fprintf (stderr, "Memory not allocated!");
        return NULL;
    }

    context = CGBitmapContextCreate (bitmapData,// 4
                                    pixelsWide,
                                    pixelsHigh,
                                    8,      // bits per component
                                    bitmapBytesPerRow,
                                    colorSpace,
                                    kCGImageAlphaPremultipliedLast);

    if (context== NULL)
    {
        free (bitmapData);// 5
        fprintf (stderr, "Context not created!");
        return NULL;
    }

    CGColorSpaceRelease( colorSpace );// 6

    return context;// 7
}

CGContextRef CreateContext(int i)
{
#if defined BITMAP
	return MyCreateBitmapContext(400, 400);
#else
	CGRect mediaBox;

	mediaBox = CGRectMake (0, 0, 400, 400);// 2
	CGContextRef cx = MyCreatePDFContext (&mediaBox, CFStringCreateWithFormat(NULL, NULL, CFSTR("output/%i.pdf"), i)); 
	CGContextBeginPage(cx, &mediaBox);
	return cx;
#endif
}

void DestroyContext(CGContextRef cx)
{
#if defined BITMAP
    void *bitmapData = CGBitmapContextGetData(cx); // 7

    CGContextRelease(cx);// 8

    if (bitmapData) free(bitmapData); 
#else
	CGContextEndPage(cx);
	CGContextRelease(cx);
#endif
}


CGMutablePathRef CreateStarburstPath(float size)
{
	CGMutablePathRef path = CGPathCreateMutable();

    const float kFullCircleInRadians    = 3.14159265358979323846 * 2.0;
    float       rotation;


	CGAffineTransform trans = CGAffineTransformMakeTranslation(size*0.5, size*0.5);

    // Draw 50 lines as a starburst
    for(    rotation = 0.0;
            rotation < kFullCircleInRadians;
            rotation += (kFullCircleInRadians/50.0) )
    {
		CGAffineTransform temp = CGAffineTransformRotate(trans, rotation);

		CGPathMoveToPoint(path, &temp, 0, 0);
		CGPathAddLineToPoint(path, &temp, size*0.5, 0);
	}

	return path;
}

CGMutablePathRef CreateShapePath(float size)
{	
	CGMutablePathRef path = CGPathCreateMutable();
	CGAffineTransform trans = CGAffineTransformMakeTranslation(size*0.5, size*0.5);

	CGPathMoveToPoint(path, &trans, 0, 0);
	CGPathAddLineToPoint(path, &trans, size*0.5, 0);
	CGPathAddLineToPoint(path, &trans, size*0.5, size*0.5);
	CGPathAddCurveToPoint(path, &trans, size*0.25, 0, -size*0.5, 0, -size*0.5, -size*0.5);
	CGPathAddLineToPoint(path, &trans, 0, -size*0.5);
	CGPathAddLineToPoint(path, &trans, 0, -size*0.25);
	CGPathAddLineToPoint(path, &trans, size*0.5, -size*0.5);
	CGPathCloseSubpath(path);

	return path;
}

void DrawShape(CGContextRef cx, float size)
{
	CGContextBeginPath(cx);
	CGContextSaveGState(cx);

	CGContextTranslateCTM(cx, size*0.5, size*0.5);

	CGContextMoveToPoint(cx, 0, 0);
	CGContextAddLineToPoint(cx, size*0.5, 0);
	CGContextAddLineToPoint(cx, size*0.5, size*0.5);
	CGContextAddCurveToPoint(cx, size*0.25, 0, -size*0.5, 0, -size*0.5, -size*0.5);
	CGContextAddLineToPoint(cx, 0, -size*0.5);
	CGContextAddLineToPoint(cx, 0, -size*0.25);
	CGContextAddLineToPoint(cx, size*0.5, -size*0.5);
	CGContextClosePath(cx);

	CGContextFillPath(cx);

	CGContextRestoreGState(cx);
}

void DrawStarburst( CGContextRef context, float size, int rep )
{
    const float kFullCircleInRadians    = 3.14159265358979323846 * 2.0;
    float       rotation;

	float mov = (rep % 50);
	mov /= 50;	
	mov *= size * 0.5;

    // Draw 50 lines as a starburst
    for(    rotation = 0.0;
            rotation < kFullCircleInRadians;
            rotation += (kFullCircleInRadians/50.0) )
    {
        // Start a new path
        CGContextBeginPath( context );
             
        // Save the graphics state so we can start with a
        // fresh rotation each time
        CGContextSaveGState( context );

        //
        // Translate the CTM to the center of the page, then
        // rotate the CTM by our current translation. Note
        // that order is important -- if you rotate the CTM
        // before the translation, the translation coordinates
        // will be rotated. In this example, we want to draw
        // in the center of the page, so we translate before
        // we rotate.
        //
        CGContextTranslateCTM( context, size * 0.5, size * 0.5 );
		CGContextTranslateCTM(context, mov, mov);
        CGContextRotateCTM( context, rotation );

        //
        // Draw a simple horizontal line
        // - First, set the current point to 0,0
        // - Second, add the horizontal line
        //      (We now have a path consisting of a horizontal line)
        // - Third, stroke the line to paint it onto the page
        //
        CGContextMoveToPoint( context, 0.0, 0.0 );
        CGContextAddLineToPoint( context, size * 0.5, 0.0 );        
   
   		CGContextStrokePath( context );

        // Restore the graphics state
        CGContextRestoreGState( context );
    }

}

void fastloop()
{
	CGPathRef path = CreateStarburstPath(400);
	CGPathRef shape = CreateShapePath(400);

	for (int i=0; i<10000; i++)
	{
		CGContextRef cx = CreateContext(i); 
		CGContextSaveGState(cx);
		float mov = (i % 50);
		mov /= 50;	
		mov *= 200 * 0.5;
		CGContextTranslateCTM(cx, mov, mov);
		CGContextAddPath(cx, shape);
		CGContextFillPath(cx);
		CGContextRestoreGState(cx);
		CGContextAddPath(cx, path);
		CGContextStrokePath(cx);
		DestroyContext(cx);
	}

	CGPathRelease(path);
}

void slowloop()
{
	for (int i=0; i<10000; i++)
	{
		CGContextRef cx = CreateContext(i); 
		DrawShape(cx, 400.0f);
		DrawStarburst(cx, 400.0f, i);
		DestroyContext(cx);
	}
}

int main()
{
	Profiler prof;
	prof.Start();
	slowloop();
	prof.Stop();
	printf("Slow Loop: %f\n", prof.Time());
	prof.Start();
	fastloop();
	prof.Stop();
	printf("Fast Loop: %f\n", prof.Time());
	return 0;
}